/**
 * This code was GENERATED using the solita package.
 * Please DO NOT EDIT THIS FILE, instead rerun solita to update it or write a wrapper to add functionality.
 *
 * See: https://github.com/metaplex-foundation/solita
 */

import * as definedTypes from '../types';
import * as web3 from '@solana/web3.js';
import * as beet from '@metaplex-foundation/beet';
import * as beetSolana from '@metaplex-foundation/beet-solana';

/**
 * Arguments used to create {@link Metadata}
 * @category Accounts
 * @category generated
 */
export type MetadataArgs = {
  key: definedTypes.Key;
  updateAuthority: web3.PublicKey;
  mint: web3.PublicKey;
  data: definedTypes.Data;
  primarySaleHappened: boolean;
  isMutable: boolean;
  editionNonce: beet.COption<number>;
  tokenStandard: beet.COption<definedTypes.TokenStandard>;
  collection: beet.COption<definedTypes.Collection>;
  uses: beet.COption<definedTypes.Uses>;
};
/**
 * Holds the data for the {@link Metadata} Account and provides de/serialization
 * functionality for that data
 *
 * @category Accounts
 * @category generated
 */
export class Metadata implements MetadataArgs {
  private constructor(
    readonly key: definedTypes.Key,
    readonly updateAuthority: web3.PublicKey,
    readonly mint: web3.PublicKey,
    readonly data: definedTypes.Data,
    readonly primarySaleHappened: boolean,
    readonly isMutable: boolean,
    readonly editionNonce: beet.COption<number>,
    readonly tokenStandard: beet.COption<definedTypes.TokenStandard>,
    readonly collection: beet.COption<definedTypes.Collection>,
    readonly uses: beet.COption<definedTypes.Uses>,
  ) {}

  /**
   * Creates a {@link Metadata} instance from the provided args.
   */
  static fromArgs(args: MetadataArgs) {
    return new Metadata(
      args.key,
      args.updateAuthority,
      args.mint,
      args.data,
      args.primarySaleHappened,
      args.isMutable,
      args.editionNonce,
      args.tokenStandard,
      args.collection,
      args.uses,
    );
  }

  /**
   * Deserializes the {@link Metadata} from the data of the provided {@link web3.AccountInfo}.
   * @returns a tuple of the account data and the offset up to which the buffer was read to obtain it.
   */
  static fromAccountInfo(accountInfo: web3.AccountInfo<Buffer>, offset = 0): [Metadata, number] {
    return Metadata.deserialize(accountInfo.data, offset);
  }

  /**
   * Retrieves the account info from the provided address and deserializes
   * the {@link Metadata} from its data.
   *
   * @throws Error if no account info is found at the address or if deserialization fails
   */
  static async fromAccountAddress(
    connection: web3.Connection,
    address: web3.PublicKey,
  ): Promise<Metadata> {
    const accountInfo = await connection.getAccountInfo(address);
    if (accountInfo == null) {
      throw new Error(`Unable to find Metadata account at ${address}`);
    }
    return Metadata.fromAccountInfo(accountInfo, 0)[0];
  }

  /**
   * Deserializes the {@link Metadata} from the provided data Buffer.
   * @returns a tuple of the account data and the offset up to which the buffer was read to obtain it.
   */
  static deserialize(buf: Buffer, offset = 0): [Metadata, number] {
    return metadataBeet.deserialize(buf, offset);
  }

  /**
   * Serializes the {@link Metadata} into a Buffer.
   * @returns a tuple of the created Buffer and the offset up to which the buffer was written to store it.
   */
  serialize(): [Buffer, number] {
    return metadataBeet.serialize(this);
  }

  /**
   * Returns the byteSize of a {@link Buffer} holding the serialized data of
   * {@link Metadata} for the provided args.
   *
   * @param args need to be provided since the byte size for this account
   * depends on them
   */
  static byteSize(args: MetadataArgs) {
    const instance = Metadata.fromArgs(args);
    return metadataBeet.toFixedFromValue(instance).byteSize;
  }

  /**
   * Fetches the minimum balance needed to exempt an account holding
   * {@link Metadata} data from rent
   *
   * @param args need to be provided since the byte size for this account
   * depends on them
   * @param connection used to retrieve the rent exemption information
   */
  static async getMinimumBalanceForRentExemption(
    args: MetadataArgs,
    connection: web3.Connection,
    commitment?: web3.Commitment,
  ): Promise<number> {
    return connection.getMinimumBalanceForRentExemption(Metadata.byteSize(args), commitment);
  }

  /**
   * Returns a readable version of {@link Metadata} properties
   * and can be used to convert to JSON and/or logging
   */
  pretty() {
    return {
      key: 'Key.' + definedTypes.Key[this.key],
      updateAuthority: this.updateAuthority.toBase58(),
      mint: this.mint.toBase58(),
      data: this.data,
      primarySaleHappened: this.primarySaleHappened,
      isMutable: this.isMutable,
      editionNonce: this.editionNonce,
      tokenStandard: this.tokenStandard,
      collection: this.collection,
      uses: this.uses,
    };
  }
}

/**
 * @category Accounts
 * @category generated
 */
export const metadataBeet = new beet.FixableBeetStruct<Metadata, MetadataArgs>(
  [
    ['key', definedTypes.keyBeet],
    ['updateAuthority', beetSolana.publicKey],
    ['mint', beetSolana.publicKey],
    ['data', definedTypes.dataBeet],
    ['primarySaleHappened', beet.bool],
    ['isMutable', beet.bool],
    ['editionNonce', beet.coption(beet.u8)],
    ['tokenStandard', beet.coption(definedTypes.tokenStandardBeet)],
    ['collection', beet.coption(definedTypes.collectionBeet)],
    ['uses', beet.coption(definedTypes.usesBeet)],
  ],
  Metadata.fromArgs,
  'Metadata',
);
